(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     18993,        689]*)
(*NotebookOutlinePosition[     19981,        718]*)
(*  CellTagsIndexPosition[     19937,        714]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[StyleBox["This is a demo file for \nNCConvexityRegion[p,{x,y}]. \
 ",
  FontColor->RGBColor[1, 0, 1]]], "Text",
  FontSize->36],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(<< \ NCGB.m;\)\), "\[IndentingNewLine]", 
    \(\(<< \ NCConvexity.m;\)\)}], "Input"],

Cell[BoxData[
    \("You have already loaded NCGB.m"\)], "Print"]
}, Open  ]],

Cell[TextData[{
  StyleBox["(1)",
    FontColor->RGBColor[1, 0, 0]],
  " Let's start with a simple example first.  Lets test the convexity of p = \
x^2."
}], "Text",
  FontSize->24],

Cell[CellGroupData[{

Cell[BoxData[
    \(NCConvexityRegion[\ x ** x, \ {x}\ ]\)], "Input",
  FontSize->14],

Cell[BoxData[
    \({{2}}\)], "Output"]
}, Open  ]],

Cell["\<\
Conclusions: We see that x^2 is matrix convex for all symmetric x, \
since the output diagonal elements {2} are positive.\
\>", "Text",
  FontSize->24],

Cell[TextData[{
  StyleBox["(2)",
    FontColor->RGBColor[1, 0, 0]],
  " Now we will test the convexity of p = x^3."
}], "Text",
  FontSize->24],

Cell[CellGroupData[{

Cell[BoxData[
    \(NCConvexityRegion[\ x ** x ** x, \ {x}\ ]\)], "Input",
  FontSize->14],

Cell[BoxData[
    \({{2\ x, \(-2\)\ inv[x]}}\)], "Output"]
}, Open  ]],

Cell["\<\
Conclusions: We see that x^3 is not matrix convex for any symmetric \
x, since the output diagonal elements {2x, -2 inv[x]} cannot both be \
positive.\
\>", "Text",
  FontSize->24],

Cell[TextData[{
  StyleBox["(3)",
    FontColor->RGBColor[1, 0, 0]],
  " What about p = x^2+y^2 ?"
}], "Text",
  FontSize->24],

Cell[CellGroupData[{

Cell[BoxData[
    \(NCConvexityRegion[\ x ** x + y ** y, \ {x, y}\ ]\)], "Input",
  FontSize->14],

Cell[BoxData[
    \({{2, 2}}\)], "Output"]
}, Open  ]],

Cell["\<\
Conclusions: We see that x^2+y^2 is matrix convex for any symmetric \
x and y, since the output diagonal elements {2,2} are positive.\
\>", "Text",
  FontSize->24],

Cell[TextData[{
  StyleBox["(4)",
    FontColor->RGBColor[1, 0, 0]],
  " What about p = x^4 ?"
}], "Text",
  FontSize->24],

Cell[CellGroupData[{

Cell[BoxData[
    \(NCConvexityRegion[\ x ** x ** x ** x, \ {x}\ ]\)], "Input",
  FontSize->14],

Cell[BoxData[
    \("L**D**tp[L] gave non-trivial blocks, so the output list is: { \
{diagonal},{subdiagonal},{-subdiagonal} }"\)], "Print"],

Cell[BoxData[
    \({{{2, 0, 0}, {0, 2}, {0, \(-2\)}}}\)], "Output"]
}, Open  ]],

Cell["\<\
Conclusions: We see that x^4 is not matrix convex for any symmetric \
x, since the eigenvalues of the D matrix include 2 and -2.  Note the \
different message and output here.  This is because D was not diagonal, but \
block diagonal. So, NCConvexityRegion gave back the diagonal, subdiagonal, \
and -subdiagonal. \
\>", "Text",
  FontSize->24],

Cell[TextData[{
  StyleBox["(5)",
    FontColor->RGBColor[1, 0, 0]],
  " What about p = x^4+y^4 ?"
}], "Text",
  FontSize->24],

Cell[CellGroupData[{

Cell[BoxData[
    \(NCConvexityRegion[\ 
      x ** x ** x ** x + y ** y ** y ** y, \ {x, y}\ ]\)], "Input",
  FontSize->14],

Cell[BoxData[
    \("L**D**tp[L] gave non-trivial blocks, so the output list is: { \
{diagonal},{subdiagonal},{-subdiagonal} }"\)], "Print"],

Cell[BoxData[
    \({{{2, 2, 0, 0, 0, 0}, {0, 0, 2, 0, 2}, {0, 0, \(-2\), 
          0, \(-2\)}}}\)], "Output"]
}, Open  ]],

Cell["\<\
Conclusions: We see that x^4+y^4 is not matrix convex for any \
symmetric x and y, since the eigenvalues of the D matrix include 2 and -2.  \
Note again the different message and output here.  This is because D was not \
diagonal, but block diagonal.  So, NCConvexityRegion gave back the diagonal, \
subdiagonal, and -subdiagonal. \
\>", "Text",
  FontSize->24],

Cell[TextData[StyleBox["What is NCConvexityRegion doing?",
  FontColor->RGBColor[0, 0, 1]]], "Text",
  FontSize->36],

Cell["\<\
Let's step through a couple of the above examples and do what \
NCConvexityRegion does to get a feeling for what is happening.\
\>", "Text",
  FontSize->24],

Cell[TextData[{
  StyleBox["(6)",
    FontColor->RGBColor[1, 0, 0]],
  " Let's go through p = x^3."
}], "Text",
  FontSize->24],

Cell[CellGroupData[{

Cell[BoxData[{
    \(hes\  = \ NCHessian[\ x ** x ** x, \ {x, h}]\), "\[IndentingNewLine]", 
    \({lt, mq, rt} = \ 
      NCMatrixOfQuadratic[hes, {h}]\), "\[IndentingNewLine]", 
    \(MatrixForm[mq]\), "\[IndentingNewLine]", 
    \(\({lf, d, rt, junk} = 
        NCLDUDecomposition[mq];\)\), "\[IndentingNewLine]", 
    \(MatrixForm[d]\), "\[IndentingNewLine]", 
    \(\)}], "Input",
  FontSize->14],

Cell[BoxData[
    \(2\ h ** h ** x + 2\ h ** x ** h + 2\ x ** h ** h\)], "Output"],

Cell[BoxData[
    \({{{h, x ** h}}, {{2\ x, 2}, {2, 0}}, {{h}, {h ** x}}}\)], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
            {\(2\ x\), "2"},
            {"2", "0"}
            }], "\[NoBreak]", ")"}],
      Function[ BoxForm`e$, 
        MatrixForm[ BoxForm`e$]]]], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
            {\(2\ x\), "0"},
            {"0", \(\(-2\)\ inv[x]\)}
            }], "\[NoBreak]", ")"}],
      Function[ BoxForm`e$, 
        MatrixForm[ BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell["\<\
Conclusions: NCConvexityRegion returns the diagonal of d.  Look at \
example (2).\
\>", "Text",
  FontSize->24],

Cell[TextData[{
  StyleBox["(7)",
    FontColor->RGBColor[1, 0, 0]],
  " Let's go through p = x^4."
}], "Text",
  FontSize->24],

Cell[CellGroupData[{

Cell[BoxData[{
    \(hes\  = \ 
      NCHessian[\ x ** x ** x ** x, \ {x, h}]\), "\[IndentingNewLine]", 
    \({lt, mq, rt} = \ 
      NCMatrixOfQuadratic[hes, {h}]\), "\[IndentingNewLine]", 
    \(MatrixForm[mq]\), "\[IndentingNewLine]", 
    \(\({lf, d, rt, junk} = 
        NCLDUDecomposition[mq];\)\), "\[IndentingNewLine]", 
    \(MatrixForm[d]\), "\[IndentingNewLine]", 
    \(\)}], "Input",
  FontSize->14],

Cell[BoxData[
    \(2\ h ** h ** x ** x + 2\ h ** x ** h ** x + 2\ h ** x ** x ** h + 
      2\ x ** h ** h ** x + 2\ x ** h ** x ** h + 
      2\ x ** x ** h ** h\)], "Output"],

Cell[BoxData[
    \({{{h, x ** h, x ** x ** h}}, {{2\ x ** x, 2\ x, 2}, {2\ x, 2, 0}, {2, 
          0, 0}}, {{h}, {h ** x}, {h ** x ** x}}}\)], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
            {\(2\ x ** x\), \(2\ x\), "2"},
            {\(2\ x\), "2", "0"},
            {"2", "0", "0"}
            }], "\[NoBreak]", ")"}],
      Function[ BoxForm`e$, 
        MatrixForm[ BoxForm`e$]]]], "Output"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
            {"2", "0", "0"},
            {"0", "0", "2"},
            {"0", "2", "0"}
            }], "\[NoBreak]", ")"}],
      Function[ BoxForm`e$, 
        MatrixForm[ BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell["\<\
Conclusions: NCConvexityRegion returns the diagonal,offdiagonal, \
and -offdiagonal of d.  Note that this answer is consistent with example (4) \
above.\
\>", "Text",
  FontSize->24],

Cell[TextData[StyleBox["What kind of options can we use?",
  FontColor->RGBColor[0, 0, 1]]], "Text",
  FontSize->36],

Cell["\<\
We can run NCConvexityRegion with AllPermutation -> True.  The \
default is false.  This will make NCConvexityRegion run NCAllPermutationLDU \
instead of NCLDUDecomposition.   When you do this you also want to set \
DiagonalSelection to the permutations you want to use.  The possible settings \
for DiagonalSelection are {n} or {k,n}.  {n} makes NCAllPermutationLDU use \
permutations 1 through n, while {k,n} makes NCAllPermutationLDU use \
permutations k through n.   Note that setting AllPermutation to True and \
selecting different permutations will give different answers.  But, they will \
all be consistent with regards to the matrix convexity of p.  When \
AllPermutation is set to true NCConvexityRegion only returns the shortest \
answer.  So although there are multiple answers depending on what \
permutations are selected, NCConvexityRegion looks through them and selects \
the shortest one to return.\
\>", "Text",
  FontSize->24],

Cell[TextData[{
  StyleBox["(8)",
    FontColor->RGBColor[1, 0, 0]],
  " Lets run x^3.  "
}], "Text",
  FontSize->24],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(hes\  = \ 
        NCHessian[\ x ** x ** x, \ {x, h}\ ];\)\), "\[IndentingNewLine]", 
    \(\({lt, mq, rt} = \ 
        NCMatrixOfQuadratic[\ hes, {h}\ ];\)\), "\[IndentingNewLine]", 
    \(MatrixForm[mq]\)}], "Input",
  FontSize->14],

Cell[BoxData[
    TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
            {\(2\ x\), "2"},
            {"2", "0"}
            }], "\[NoBreak]", ")"}],
      Function[ BoxForm`e$, 
        MatrixForm[ BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell["\<\
We have a 2x2 matrix.  So the number of permutations is only \
2.\
\>", "Text",
  FontSize->24],

Cell[CellGroupData[{

Cell[BoxData[
    \(NCConvexityRegion[\ x ** x ** x, \ {x}, AllPermutation \[Rule] True, 
      DiagonalSelection \[Rule] {1}\ ]\)], "Input",
  FontSize->14],

Cell[BoxData[
    InterpretationBox[\("Middle matrix is size "\[InvisibleSpace]2\
\[InvisibleSpace]"X"\[InvisibleSpace]2\),
      SequenceForm[ "Middle matrix is size ", 2, "X", 2],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("At most "\[InvisibleSpace]2\[InvisibleSpace]" \
permutations possible."\),
      SequenceForm[ "At most ", 2, " permutations possible."],
      Editable->False]], "Print"],

Cell[BoxData[
    \({1}\)], "Print"],

Cell[BoxData[
    \({{2\ x, \(-2\)\ inv[x]}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(NCConvexityRegion[\ x ** x ** x, \ {x}, AllPermutation \[Rule] True, 
      DiagonalSelection \[Rule] {2, 2}\ ]\)], "Input",
  FontSize->14],

Cell[BoxData[
    InterpretationBox[\("Middle matrix is size "\[InvisibleSpace]2\
\[InvisibleSpace]"X"\[InvisibleSpace]2\),
      SequenceForm[ "Middle matrix is size ", 2, "X", 2],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("At most "\[InvisibleSpace]2\[InvisibleSpace]" \
permutations possible."\),
      SequenceForm[ "At most ", 2, " permutations possible."],
      Editable->False]], "Print"],

Cell[BoxData[
    \({2}\)], "Print"],

Cell[BoxData[
    \("Permutation Disregarded 1"\)], "Print"],

Cell[BoxData[
    \({{2\ x, \(-2\)\ inv[x]}}\)], "Output"]
}, Open  ]],

Cell["\<\
Conclusions: In this case we get the same answer even though we \
used different permutations. \
\>", "Text",
  FontSize->24],

Cell[TextData[{
  StyleBox["(9)",
    FontColor->RGBColor[1, 0, 0]],
  " Lets run x^4.  "
}], "Text",
  FontSize->24],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(hes\  = \ 
        NCHessian[\ 
          x ** x ** x ** x, \ {x, h}\ ];\)\), "\[IndentingNewLine]", 
    \(\({lt, mq, rt} = \ 
        NCMatrixOfQuadratic[\ hes, {h}\ ];\)\), "\[IndentingNewLine]", 
    \(MatrixForm[mq]\)}], "Input",
  FontSize->14],

Cell[BoxData[
    TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
            {\(2\ x ** x\), \(2\ x\), "2"},
            {\(2\ x\), "2", "0"},
            {"2", "0", "0"}
            }], "\[NoBreak]", ")"}],
      Function[ BoxForm`e$, 
        MatrixForm[ BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell["\<\
We have a 3x3 matrix.  So the number of permutations is only \
12.\
\>", "Text",
  FontSize->24],

Cell[CellGroupData[{

Cell[BoxData[
    \(NCConvexityRegion[\ x ** x ** x ** x, \ {x}, 
      AllPermutation \[Rule] True, 
      DiagonalSelection \[Rule] {1, 2}\ ]\)], "Input",
  FontSize->14],

Cell[BoxData[
    InterpretationBox[\("Middle matrix is size "\[InvisibleSpace]3\
\[InvisibleSpace]"X"\[InvisibleSpace]3\),
      SequenceForm[ "Middle matrix is size ", 3, "X", 3],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("At most "\[InvisibleSpace]12\[InvisibleSpace]" \
permutations possible."\),
      SequenceForm[ "At most ", 12, " permutations possible."],
      Editable->False]], "Print"],

Cell[BoxData[
    \({1, 2}\)], "Print"],

Cell[BoxData[
    \("Permutation Disregarded 1"\)], "Print"],

Cell[BoxData[
    \({{2\ x ** x, \(-2\)\ inv[x] ** inv[x], 2}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(NCConvexityRegion[\ x ** x ** x ** x, \ {x}, 
      AllPermutation \[Rule] True, 
      DiagonalSelection \[Rule] {1, 12}\ ]\)], "Input",
  FontSize->14],

Cell[BoxData[
    InterpretationBox[\("Middle matrix is size "\[InvisibleSpace]3\
\[InvisibleSpace]"X"\[InvisibleSpace]3\),
      SequenceForm[ "Middle matrix is size ", 3, "X", 3],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("At most "\[InvisibleSpace]12\[InvisibleSpace]" \
permutations possible."\),
      SequenceForm[ "At most ", 12, " permutations possible."],
      Editable->False]], "Print"],

Cell[BoxData[
    \({1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12}\)], "Print"],

Cell[BoxData[
    \("Permutation Disregarded 1"\)], "Print"],

Cell[BoxData[
    \("Permutation Disregarded 1"\)], "Print"],

Cell[BoxData[
    \("Permutation Disregarded 7"\)], "Print"],

Cell[BoxData[
    \("Permutation Disregarded 7"\)], "Print"],

Cell[BoxData[
    \("Permutation Disregarded 7"\)], "Print"],

Cell[BoxData[
    \("Permutation Disregarded 7"\)], "Print"],

Cell[BoxData[
    \("L**D**tp[L] gave non-trivial blocks, so the output list is: { \
{diagonal},{subdiagonal},{-subdiagonal} }"\)], "Print"],

Cell[BoxData[
    \({{{2, 0, 0}, {0, 2}, {0, \(-2\)}}}\)], "Output"]
}, Open  ]],

Cell["\<\
Conclusions: When we tried using permutations 1 through 2 we got a \
different answer than when we used all the permutations.  But the answers in \
terms of convexity are the same.  x^4 is not matrix convex. \
\>", "Text",
  FontSize->24],

Cell["WARNINGS", "Text",
  FontSize->72,
  FontColor->RGBColor[1, 0, 0]],

Cell["The following example appears to give misinformation.", "Text",
  FontSize->24],

Cell[TextData[{
  StyleBox["(10)",
    FontColor->RGBColor[1, 0, 0]],
  " Giving NCConvexityRegion polys that aren't symmetric."
}], "Text",
  FontSize->24],

Cell[CellGroupData[{

Cell[BoxData[
    \(NCConvexityRegion[1 + x ** y, {x, y}]\)], "Input",
  FontSize->14],

Cell[BoxData[
    \({{2}}\)], "Output"]
}, Open  ]],

Cell["\<\
1+xy is not matrix convex everywhere.  What happened here?  We gave \
NCConvexityRegion a poly that isn't symmetric.  NCConvexityRegion only works \
with symmetric polys.  If we give it the symmetric version we get:\
\>", \
"Text",
  FontSize->24],

Cell[CellGroupData[{

Cell[BoxData[
    \(NCConvexityRegion[1 + x ** y + y ** x, {x, y}]\)], "Input",
  FontSize->14],

Cell[BoxData[
    \("L**D**tp[L] gave non-trivial blocks, so the output list is: { \
{diagonal},{subdiagonal},{-subdiagonal} }"\)], "Print"],

Cell[BoxData[
    \({{{0, 0}, {2}, {\(-2\)}}}\)], "Output"]
}, Open  ]],

Cell["This is correct.  1+xy+yx is not matrix convex anywhere.", "Text",
  FontSize->24],

Cell[TextData[{
  StyleBox["(11)",
    FontColor->RGBColor[1, 0, 0]],
  " NCConvexityRegion cannot handle commutative variables.  "
}], "Text",
  FontSize->24],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(SetCommutative[A, B, F, G];\)\), "\[IndentingNewLine]", 
    \(NCConvexityRegion[\((A + B*x)\) ** inv[F + G*x] + 
        tp[\ \((A + B*x)\) ** inv[F + G*x]\ ], {x, y}]\)}], "Input",
  FontSize->14],

Cell[BoxData[
    \("Possible Problem in symmetry check"\)], "Print"],

Cell[BoxData[
    \("Possible Problem in symmetry check"\)], "Print"],

Cell[BoxData[
    \("Possible error in symmetry!!!"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Possible trouble in last "\[InvisibleSpace]3\
\[InvisibleSpace]" elements."\),
      SequenceForm[ "Possible trouble in last ", 3, " elements."],
      Editable->False]], "Print"],

Cell[BoxData[
    \({{2\ B\ G\^2\ inv[F + G\ x], 2\ B\ G\^2\ inv[F + G\ tp[x]], 0, 
        0}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(NCConvexityRegion[A*x ** x ** x ** x ** x, \ {x}]\)], "Input",
  FontSize->14],

Cell[BoxData[
    \({{2\ A\ x, \(-2\)\ A\^2\ inv[A, x], 
        2\ A\ x ** x ** x + 
          1\/2\ \((2\ A\ x - 2\ A\^2\ x ** x ** inv[A, x])\) ** 
              inv[A\^2, 
                inv[A, x]] ** \((2\ A\ x - 2\ A\^2\ inv[A, x] ** x ** x)\) - 
          2\ A\^2\ x ** x ** inv[A, x] ** x ** x, \(-4\)\ A\^2\ inv[
            2\ A\ x ** x ** x + 
              1\/2\ \((2\ A\ x - 2\ A\^2\ x ** x ** inv[A, x])\) ** 
                  inv[A\^2, 
                    inv[A, x]] ** \((2\ A\ x - 
                      2\ A\^2\ inv[A, x] ** x ** x)\) - 
              2\ A\^2\ x ** x ** inv[A, x] ** x ** x]}}\)], "Output"]
}, Open  ]],

Cell["\<\
In both of the above examples NCConvexityRegion had problems.  In \
the first we get symmetry errors, although the poly is symmetric.  In the \
second we get weird inverses.  So don't feed NCConvexityRegion commutative \
variables.\
\>", "Text",
  FontSize->24],

Cell[TextData[{
  StyleBox["(12)",
    FontColor->RGBColor[1, 0, 0]],
  " Testing linear functions with NCConvexityRegion.  "
}], "Text",
  FontSize->24],

Cell[CellGroupData[{

Cell[BoxData[
    \(NCConvexityRegion[1 + x, \ {x}]\)], "Input",
  FontSize->14],

Cell[BoxData[
    \({{0}}\)], "Output"]
}, Open  ]],

Cell["\<\
A linear function is convex everywhere, but strictly convex \
nowhere.  This is reflected in the output 0.\
\>", "Text",
  FontSize->24]
},
FrontEndVersion->"4.1 for X",
ScreenRectangle->{{0, 1280}, {0, 1024}},
WindowSize->{1186, 875},
WindowMargins->{{Automatic, 24}, {Automatic, 2}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "toal", \
"SHAHEEN", "Summer02", "Convexity", "NewStuff", "NEWNCCONVEXITYWITHDOC"}, \
"NCConvexityRegionDemo.nb.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1705, 50, 141, 3, 110, "Text"],

Cell[CellGroupData[{
Cell[1871, 57, 108, 2, 43, "Input"],
Cell[1982, 61, 65, 1, 23, "Print"]
}, Open  ]],
Cell[2062, 65, 181, 6, 44, "Text"],

Cell[CellGroupData[{
Cell[2268, 75, 85, 2, 29, "Input"],
Cell[2356, 79, 39, 1, 27, "Output"]
}, Open  ]],
Cell[2410, 83, 161, 4, 74, "Text"],
Cell[2574, 89, 144, 5, 44, "Text"],

Cell[CellGroupData[{
Cell[2743, 98, 90, 2, 29, "Input"],
Cell[2836, 102, 58, 1, 27, "Output"]
}, Open  ]],
Cell[2909, 106, 190, 5, 74, "Text"],
Cell[3102, 113, 126, 5, 44, "Text"],

Cell[CellGroupData[{
Cell[3253, 122, 97, 2, 29, "Input"],
Cell[3353, 126, 42, 1, 27, "Output"]
}, Open  ]],
Cell[3410, 130, 173, 4, 74, "Text"],
Cell[3586, 136, 122, 5, 44, "Text"],

Cell[CellGroupData[{
Cell[3733, 145, 95, 2, 29, "Input"],
Cell[3831, 149, 140, 2, 23, "Print"],
Cell[3974, 153, 68, 1, 27, "Output"]
}, Open  ]],
Cell[4057, 157, 354, 7, 104, "Text"],
Cell[4414, 166, 126, 5, 44, "Text"],

Cell[CellGroupData[{
Cell[4565, 175, 124, 3, 29, "Input"],
Cell[4692, 180, 140, 2, 23, "Print"],
Cell[4835, 184, 111, 2, 27, "Output"]
}, Open  ]],
Cell[4961, 189, 371, 7, 104, "Text"],
Cell[5335, 198, 116, 2, 62, "Text"],
Cell[5454, 202, 166, 4, 74, "Text"],
Cell[5623, 208, 127, 5, 44, "Text"],

Cell[CellGroupData[{
Cell[5775, 217, 401, 9, 124, "Input"],
Cell[6179, 228, 82, 1, 27, "Output"],
Cell[6264, 231, 87, 1, 27, "Output"],
Cell[6354, 234, 232, 7, 58, "Output"],
Cell[6589, 243, 247, 7, 58, "Output"]
}, Open  ]],
Cell[6851, 253, 121, 4, 44, "Text"],
Cell[6975, 259, 127, 5, 44, "Text"],

Cell[CellGroupData[{
Cell[7127, 268, 413, 10, 124, "Input"],
Cell[7543, 280, 177, 3, 27, "Output"],
Cell[7723, 285, 154, 2, 27, "Output"],
Cell[7880, 289, 286, 8, 76, "Output"],
Cell[8169, 299, 266, 8, 76, "Output"]
}, Open  ]],
Cell[8450, 310, 192, 5, 74, "Text"],
Cell[8645, 317, 116, 2, 62, "Text"],
Cell[8764, 321, 956, 15, 284, "Text"],
Cell[9723, 338, 117, 5, 44, "Text"],

Cell[CellGroupData[{
Cell[9865, 347, 257, 6, 67, "Input"],
Cell[10125, 355, 232, 7, 58, "Output"]
}, Open  ]],
Cell[10372, 365, 105, 4, 44, "Text"],

Cell[CellGroupData[{
Cell[10502, 373, 157, 3, 29, "Input"],
Cell[10662, 378, 215, 4, 23, "Print"],
Cell[10880, 384, 210, 4, 23, "Print"],
Cell[11093, 390, 36, 1, 23, "Print"],
Cell[11132, 393, 58, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11227, 399, 160, 3, 29, "Input"],
Cell[11390, 404, 215, 4, 23, "Print"],
Cell[11608, 410, 210, 4, 23, "Print"],
Cell[11821, 416, 36, 1, 23, "Print"],
Cell[11860, 419, 60, 1, 23, "Print"],
Cell[11923, 422, 58, 1, 27, "Output"]
}, Open  ]],
Cell[11996, 426, 135, 4, 44, "Text"],
Cell[12134, 432, 117, 5, 44, "Text"],

Cell[CellGroupData[{
Cell[12276, 441, 273, 7, 67, "Input"],
Cell[12552, 450, 286, 8, 76, "Output"]
}, Open  ]],
Cell[12853, 461, 106, 4, 44, "Text"],

Cell[CellGroupData[{
Cell[12984, 469, 172, 4, 29, "Input"],
Cell[13159, 475, 215, 4, 23, "Print"],
Cell[13377, 481, 212, 4, 23, "Print"],
Cell[13592, 487, 39, 1, 23, "Print"],
Cell[13634, 490, 60, 1, 23, "Print"],
Cell[13697, 493, 76, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13810, 499, 173, 4, 29, "Input"],
Cell[13986, 505, 215, 4, 23, "Print"],
Cell[14204, 511, 212, 4, 23, "Print"],
Cell[14419, 517, 72, 1, 23, "Print"],
Cell[14494, 520, 60, 1, 23, "Print"],
Cell[14557, 523, 60, 1, 23, "Print"],
Cell[14620, 526, 60, 1, 23, "Print"],
Cell[14683, 529, 60, 1, 23, "Print"],
Cell[14746, 532, 60, 1, 23, "Print"],
Cell[14809, 535, 60, 1, 23, "Print"],
Cell[14872, 538, 140, 2, 23, "Print"],
Cell[15015, 542, 68, 1, 27, "Output"]
}, Open  ]],
Cell[15098, 546, 248, 5, 74, "Text"],
Cell[15349, 553, 72, 2, 104, "Text"],
Cell[15424, 557, 85, 1, 44, "Text"],
Cell[15512, 560, 156, 5, 44, "Text"],

Cell[CellGroupData[{
Cell[15693, 569, 86, 2, 29, "Input"],
Cell[15782, 573, 39, 1, 27, "Output"]
}, Open  ]],
Cell[15836, 577, 257, 6, 74, "Text"],

Cell[CellGroupData[{
Cell[16118, 587, 95, 2, 29, "Input"],
Cell[16216, 591, 140, 2, 23, "Print"],
Cell[16359, 595, 59, 1, 27, "Output"]
}, Open  ]],
Cell[16433, 599, 88, 1, 44, "Text"],
Cell[16524, 602, 159, 5, 44, "Text"],

Cell[CellGroupData[{
Cell[16708, 611, 221, 4, 48, "Input"],
Cell[16932, 617, 69, 1, 23, "Print"],
Cell[17004, 620, 69, 1, 23, "Print"],
Cell[17076, 623, 64, 1, 23, "Print"],
Cell[17143, 626, 218, 4, 23, "Print"],
Cell[17364, 632, 109, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17510, 639, 98, 2, 29, "Input"],
Cell[17611, 643, 628, 12, 80, "Output"]
}, Open  ]],
Cell[18254, 658, 271, 6, 104, "Text"],
Cell[18528, 666, 153, 5, 44, "Text"],

Cell[CellGroupData[{
Cell[18706, 675, 80, 2, 29, "Input"],
Cell[18789, 679, 39, 1, 27, "Output"]
}, Open  ]],
Cell[18843, 683, 146, 4, 44, "Text"]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

